# ----------------------------------------------------------------------------
# - std-adp-texpg                                                            -
# - afnix:adp latex page module                                              -
# ----------------------------------------------------------------------------
# - This program is  free software;  you can  redistribute it and/or  modify -
# - it provided that this copyright notice is kept intact.                   -
# -                                                                          -
# - This  program  is  distributed in the hope  that it  will be useful, but -
# - without  any   warranty;  without  even   the   implied    warranty   of -
# - merchantability  or fitness for a particular purpose. In not event shall -
# - the copyright holder be  liable for  any direct, indirect, incidental or -
# - special damages arising in any way out of the use of this software.      -
# ----------------------------------------------------------------------------
# - copyright (c) 1999-2009 amaury darsch                                    -
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# - global section                                                           -
# ----------------------------------------------------------------------------

# define the writer class
const afnix:adp:texpg (class)
# define the writer nameset
const AFNIX:ADP:TEXPG (nameset AFNIX:ADP)

# ----------------------------------------------------------------------------
# - private section                                                          -
# ----------------------------------------------------------------------------

# the page commands
const AFNIX:ADP:TEXPG:NRM-CHAP-CMD "chapter"
const AFNIX:ADP:TEXPG:NRM-SECT-CMD "section"
const AFNIX:ADP:TEXPG:NRM-SUBS-CMD "subsection"

const AFNIX:ADP:TEXPG:ENV-ITMZ-CMD "itemize"
const AFNIX:ADP:TEXPG:NRM-ITEM-CMD "item"

# ----------------------------------------------------------------------------
# - initial section                                                          -
# ----------------------------------------------------------------------------

# preset the writer environment

trans afnix:adp:texpg:preset nil {
  # set the header name
  trans this:hnam (String)
  # create the command vector
  const this:cvec (Vector)
}

# ----------------------------------------------------------------------------
# - method section                                                           -
# ----------------------------------------------------------------------------

# write the page content to a stream
# @param os the output stream to write

trans afnix:adp:texpg:write (os) {
  # write the header
  os:writeln "% This  documentation, a part  of  the  afnix programming"
  os:writeln "% language distribution is free and can be distributed as"
  os:writeln "% long as this copyright notice is left intact."
  os:writeln "%"
  os:writeln "% This documentation is distributed in the hope  that it"
  os:writeln "% will be useful, but without any warranty; without even"
  os:writeln "% the implied warranty  of  merchantability and  fitness"
  os:writeln "% for  a  particular  purpose.  In  no  event, shall the"
  os:writeln "% copyright holder be  liable  for  any direct, incident"
  os:writeln "% incidental or special damages arising in any way out"
  os:writeln "% of the use of this documentation."
  os:writeln "%"
  os:writeln "% Copyright (c) 1999-2009 Amaury Darsch"
  # write the commands
  for (c) (this:cvec) (c:write os)
}

# add the page header by name section and text
# @param name the header name
# @param snum the section number
# @param desc the header description

trans afnix:adp:texpg:add-head (name snum desc) {
  const this:hnam (name:clone)
  const this:hnum (snum:clone)
  const this:hdes (desc:clone)
}

# add the page title
# @param title the title to set

trans afnix:adp:texpg:add-title (title) {
  const this:htit (title:clone)
}

# add a chapter to the page
# @param title the chapter title

trans afnix:adp:texpg:add-chapter (title) {
  # create a new chapter command
  const tsc (afnix:adp:texcc AFNIX:ADP:TEXPG:NRM-CHAP-CMD title)
  # add the command
  this:cvec:add tsc
}

# add a section to the page
# @param title the section title

trans afnix:adp:texpg:add-section (title) {
  # create a new section command
  const tsc (afnix:adp:texcc AFNIX:ADP:TEXPG:NRM-SECT-CMD title)
  # add the command
  this:cvec:add tsc
}

# add a subsection to the page
# @param title the subsection title

trans afnix:adp:texpg:add-subsect (title) {
  # create a new subsection command
  const tsc (afnix:adp:texcc AFNIX:ADP:TEXPG:NRM-SUBS-CMD title)
  # add the command
  this:cvec:add tsc
}

# add a new paragraph

trans afnix:adp:texpg:new-para nil {
  # create a new text paragarph
  const para (afnix:adp:texpc)
  # add the command
  this:cvec:add para
  # here we are
  eval para
}

# add an example by text
# @param text the example text

trans afnix:adp:texpg:add-example (text) {
  # prepare text data
  const data (afnix:txt:indent (text:strip " \t\r\n") "{(" "})" "  ")
  # create the example command
  const expl (afnix:adp:texec data)
  # add the command
  this:cvec:add expl
}

# begin a new list by starting a relative indentation

trans afnix:adp:texpg:beg-list nil {
  # start an itemize command
  this:cvec:add (afnix:adp:texcc "begin" AFNIX:ADP:TEXPG:ENV-ITMZ-CMD)
}

# end a list by removing the relative indentation

trans afnix:adp:texpg:end-list nil {
  # end an itemize command
  this:cvec:add (afnix:adp:texcc "end" AFNIX:ADP:TEXPG:ENV-ITMZ-CMD)
}

# add a list item by reference and text
# @param iref the item reference 
# @param item the item text

trans afnix:adp:texpg:add-list-item (iref text) {
  # create item command
  const itmc (afnix:adp:texsc AFNIX:ADP:TEXPG:NRM-ITEM-CMD)
  this:cvec:add itmc
  # create the data text
  const data (+ (+ "\\textbf{" iref) "}\\\\\n")
  data:+= (text:strip " \t\r\n")
  # set the command data
  itmc:set-data data
}

# add a list item by text
# @param iref the item reference 
# @param item the item text

trans afnix:adp:texpg:add-list-elem (text) {
  # create item command
  const itmc (afnix:adp:texsc AFNIX:ADP:TEXPG:NRM-ITEM-CMD)
  this:cvec:add itmc
  # set the command data
  itmc:set-data (text:strip " \t\r\n")
}

# add a new table
# @param clen the number of columns

trans afnix:adp:texpg:new-tabl (clen) {
  # create a new table command
  const tabl (afnix:adp:textb clen)
  # add the command
  this:cvec:add tabl
  # here we are
  eval tabl
}
