// ---------------------------------------------------------------------------
// - Blkmsg.cpp                                                              -
// - afnix:sec module - base message blkmsg class implementation             -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2009 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Blkmsg.hpp"

namespace afnix {

  // -------------------------------------------------------------------------
  // - class section                                                         -
  // -------------------------------------------------------------------------

  // create a block message by name and size

  Blkmsg::Blkmsg (const String& name, const long size) {
    // check for size
    if (size <= 0) {
      throw Exception ("size-error", 
		       "invalid size in block message constructor");
    }
    // initialize blkmsg
    d_name = name;
    d_size = size;
    p_data = new t_byte[d_size];
    reset ();
  }

  // destroy this blkmsg
  
  Blkmsg::~Blkmsg (void) {
    delete [] p_data;    
  }

  // return the class name

  String Blkmsg::repr (void) const {
    return "Blkmsg";
  }

  // return the block buffer name
  
  String Blkmsg::getname (void) const {
    rdlock ();
    try {
      String result = d_name;
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  }
  
  // reset this block message buffer

  void Blkmsg::reset (void) {
    wrlock ();
    try {
      d_count = 0;
      d_llen  = 0;
      d_ulen  = 0;
      for (long i = 0; i < d_size; i++) p_data[i] = nilc;
      unlock ();
    } catch (...) {
      unlock ();
      throw;
    }
  }

  // clear the block message buffer counter

  void Blkmsg::clear (void) {
    wrlock ();
    try {
      d_count = 0;
      unlock ();
    } catch (...) {
      unlock ();
      throw;
    }
  }

  // return the block message size

  long Blkmsg::getsize (void) const {
    rdlock ();
    try {
      long result = d_size;
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  }

  // get the block message counter
  
  long Blkmsg::getcount (void) const {
    rdlock ();
    try {
      long result = d_count;
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  }

  // get the block message lower length
  
  t_octa Blkmsg::getllen (void) const {
    rdlock ();
    try {
      t_octa result = d_llen;
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  }

  // get the block message upper length
  
  t_octa Blkmsg::getulen (void) const {
    rdlock ();
    try {
      t_octa result = d_ulen;
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  }
   
  // copy some bytes in the buffer and return the number of copied data
  
  long Blkmsg::copy (const t_byte* data, const long size) {
    wrlock ();
    try {
      long result = 0;
      while ((d_count < d_size) && (result < size)) {
	p_data[d_count++] = data[result++];
	if (d_llen == 0xFFFFFFFFFFFFFFFFULL) {
	  d_ulen++;
	  d_llen = 0;
	} else {
	  d_llen++;
	}
      }
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  } 
  
  // copy some bytes in the buffer from a stream
  
  long Blkmsg::copy (Buffer& buf) {
    wrlock ();
    try {
      long result = 0;
      while (buf.empty () == false) {
	p_data[d_count++] = buf.read ();
	if (d_llen == 0xFFFFFFFFFFFFFFFFULL) {
	  d_ulen++;
	  d_llen = 0;
	} else {
	  d_llen++;
	}
	result++;
      }
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  }

  // copy some bytes in the buffer from a stream
  
  long Blkmsg::copy (InputStream& is) {
    wrlock ();
    try {
      long result = 0;
      while (is.valid () == true) {
	p_data[d_count++] = is.read ();
	if (d_llen == 0xFFFFFFFFFFFFFFFFULL) {
	  d_ulen++;
	  d_llen = 0;
	} else {
	  d_llen++;
	}
	result++;
      }
      unlock ();
      return result;
    } catch (...) {
      unlock ();
      throw;
    }
  }
}
